@@ -35,7 +35,7 @@ from metview import plotting
 from metview.metviewpy.ipython import is_ipython_active, import_widgets
 from metview.metviewpy import utils
 
-__version__ = "1.10.2"
+__version__ = "1.12.0"
 
 
 # logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(message)s")
@@ -292,7 +292,6 @@ class Request(dict, Value):
             self.verb = myverb
         if isinstance(req, dict):
             self.create_new(self.verb, req)
-            self.update(req)  # copy into dict
             return
 
         # initialise from a Macro pointer
@@ -380,7 +379,9 @@ class Request(dict, Value):
             push_arg(new_key)
             push_arg(new_val)
             lib.p_set_subvalue_from_arg_stack(self.val_pointer)
-        dict.__setitem__(self, index, value)
+            dict.__setitem__(self, new_key, new_val)
+        else:
+            dict.__setitem__(self, index, value)
 
 
 def push_bytes(b):
@@ -465,6 +466,17 @@ def valid_date(*args, base=None, step=None, step_units=None):
         return [base + step_units * int(x) for x in step]
 
 
+def sort(*args, **kwargs):
+    if len(args) != 0 and isinstance(args[0], Fieldset):
+        if len(args) == 1:
+            return args[0].sort(**kwargs)
+        else:
+            args = args[1:]
+            return args[0].sort(*args, **kwargs)
+    else:
+        return call("sort", *args)
+
+
 class File(Value):
     def __init__(self, val_pointer):
         Value.__init__(self, val_pointer)
@@ -546,9 +558,15 @@ class FileBackedValueWithOperators(FileBackedValue):
     def __and__(self, other):
         return met_and(self, other)
 
+    def __rand__(self, other):
+        return met_and(other, self)
+
     def __or__(self, other):
         return met_or(self, other)
 
+    def __ror__(self, other):
+        return met_or(other, self)
+
     def __invert__(self):
         return met_not(self)
 
@@ -648,7 +666,7 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
             support_slicing=True,
         )
         self._db = None
-        self._param_info = None
+        self._ds_param_info = None
         self._label = ""
 
         if (path is not None) and (fields is not None):
@@ -698,31 +716,32 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
             self._db = FieldsetDb(fs=self)
             self._db.scan()
 
-    def select(self, *args, **kwargs):
+    def _get_db(self):
         if self._db is None:
             self._db = FieldsetDb(fs=self)
-        # LOG.debug(f"kwargs={kwargs}")
         assert self._db is not None
+        return self._db
+
+    def select(self, *args, **kwargs):
         if len(args) == 1 and isinstance(args[0], dict):
-            return self._db.select(**args[0])
+            return self._get_db().select(**args[0])
         else:
-            return self._db.select(**kwargs)
+            return self._get_db().select(**kwargs)
 
     def describe(self, *args, **kwargs):
-        if self._db is None:
-            self._db = FieldsetDb(fs=self)
-        return self._db.describe(*args, **kwargs)
+        return self._get_db().describe(*args, **kwargs)
 
     def ls(self, **kwargs):
-        if self._db is None:
-            self._db = FieldsetDb(fs=self)
-        return self._db.ls(**kwargs)
+        return self._get_db().ls(**kwargs)
+
+    def sort(self, *args, **kwargs):
+        return self._get_db().sort(*args, **kwargs)
 
     @property
-    def param_info(self):
-        if self._param_info is None:
-            self._param_info = FieldsetDb.make_param_info(self)
-        return self._param_info
+    def ds_param_info(self):
+        if self._ds_param_info is None:
+            self._ds_param_info = FieldsetDb.make_param_info(self)
+        return self._ds_param_info
 
     @property
     def label(self):
@@ -746,35 +765,42 @@ class Fieldset(FileBackedValueWithOperators, ContainerValue):
 
         self._label = value
 
-    def unique(self, key):
-        if self._db:
-            return self._db.unique(key)
-        else:
-            return []
+    def _unique_metadata(self, key):
+        return self._get_db().unique(key)
 
-    def style(self, plot_type="map"):
+    def ds_style(self, plot_type="map"):
         from metview import style
 
         return style.get_db().style(self, plot_type=plot_type)
 
-    def style_list(self, plot_type="map"):
+    def ds_style_list(self, plot_type="map"):
         from metview import style
 
         return style.get_db().style_list(self, plot_type=plot_type)
 
-    def speed(self):
-        if self._db is not None:
-            fs = self._db.speed()
-            return fs
+    def speed(self, *args):
+        if len(args) == 0:
+            u = self[0::2]
+            v = self[1::2]
+            if len(u) != len(v):
+                raise Exception(
+                    f"Fieldsets must contain an even number of fields for this operation! len={len(self)} is not even!"
+                )
+            sp = u.speed(v)
+            sp._init_db_from(self)
+            return sp
         else:
-            return None
+            return call("speed", self, *args)
 
-    def deacc(self, skip_first=None):
-        if self._db is None:
-            self._db = FieldsetDb(fs=self)
-        assert self._db is not None
-        fs = self._db.deacc(skip_first=skip_first)
-        return fs
+    def deacc(self, **kwargs):
+        r = utils.deacc(self, **kwargs)
+        r._init_db_from(self)
+        return r
+
+    def _init_db_from(self, other):
+        if self._db is None and other._db is not None:
+            self._db = FieldsetDb(self, other.label)
+            self._db.load()
 
     def __getitem__(self, key):
         if isinstance(key, str):
@@ -1238,6 +1264,7 @@ def bind_functions(namespace, module_name=None):
     namespace["merge"] = merge
     namespace["dataset_to_fieldset"] = dataset_to_fieldset
     namespace["valid_date"] = valid_date
+    namespace["sort"] = sort
     namespace["load_dataset"] = Dataset.load_dataset
     namespace["plot_maps"] = plotting.plot_maps
     namespace["plot_diff_maps"] = plotting.plot_diff_maps
