@@ -64,6 +64,16 @@ class CodesHandle:
     #    s = "CodesHandle("
     #    return s + str(self.handle) + "," + self.path + "," + str(self.offset) + ")"
 
+    def call_get_func(self, func, key, default=False):
+        try:
+            result = func(self.handle, key)
+        except Exception as exp:
+            if default != False:
+                return default
+            else:
+                raise exp
+        return result
+
     def get_any(self, keys, key_type=None):
         # single key with key_type specified
         if key_type is not None:
@@ -84,7 +94,7 @@ class CodesHandle:
                     ):
                         key_type_str = "na"
                 func = CodesHandle._STR_TYPES.get(key_type_str, None)[0]
-                result.append(func(self.handle, key))
+                result.append(self.call_get_func(func, key, default=None))
             return result
 
     def get_string(self, key):
@@ -480,7 +490,8 @@ class Fieldset:
             #     # if isinstance(value, float):
             #     #    return int(value)
             #     return value
-
+            elif isinstance(index, str):
+                return self._get_db().select_with_name(index)
             else:
                 return Fieldset(fields=self._always_list(self.fields[index]))
         except IndexError as ide:
@@ -747,6 +758,9 @@ class Fieldset:
         assert self._db is not None
         return self._db
 
+    def _unique_metadata(self, key):
+        return self._get_db().unique(key)
+
     def select(self, *args, **kwargs):
         if len(args) == 1 and isinstance(args[0], dict):
             return self._get_db().select(**args[0])
@@ -759,40 +773,48 @@ class Fieldset:
     def ls(self, **kwargs):
         return self._get_db().ls(**kwargs)
 
-    def deacc(self, skip_first=False, mark_derived=False):
-        if len(self.fields) > 1:
-            v = self[1:] - self[:-1]
-            if not skip_first:
-                r = self[0] * 0
-                r = r.merge(v)
-            else:
-                r = v
-            if not mark_derived:
-                r = r.grib_set_long(["generatingProcessIdentifier", 148])
-            return r
+    def sort(self, *args, **kwargs):
+        return self._get_db().sort(*args, **kwargs)
 
-    def speed(self, other):
-        result = Fieldset(temporary=True)
-        param_ids = {
-            131: 10,  # atmospheric wind
-            165: 207,  # 10m wind
-            228246: 228249,  # 100m wind
-            228239: 228241,  # 200m wind
-        }
-        with open(result.temporary.path, "wb") as fout:
-            for f, g in zip(self.fields, other.fields):
-                sp = np.sqrt(np.square(f.values()) + np.square(g.values()))
-                c = f.clone()
-                c.encode_values(sp)
-                param_id_u = f.grib_get("paramId", CodesHandle.LONG)
-                param_id_sp = param_ids.get(param_id_u, None)
-                if param_id_sp is not None:
-                    c = c.grib_set(["paramId", param_id_sp], CodesHandle.LONG)
-                result._append_field(c)
-                result.fields[-1].write(
-                    fout, result.temporary.path, temp=result.temporary
+    def deacc(self, **kwargs):
+        return utils.deacc(self, **kwargs)
+
+    def speed(self, *args):
+        if len(args) == 0:
+            u = self[0::2]
+            v = self[1::2]
+            if len(u) != len(v):
+                raise Exception(
+                    f"Fieldsets must contain an even number of fields for this operation! len={len(self.fields)} is not even!"
                 )
-        return result
+            return u.speed(v)
+        elif len(args) == 1:
+            other = args[0]
+            result = Fieldset(temporary=True)
+            param_ids = {
+                131: 10,  # atmospheric wind
+                165: 207,  # 10m wind
+                228246: 228249,  # 100m wind
+                228239: 228241,  # 200m wind
+            }
+            if len(self.fields) != len(other):
+                raise Exception(
+                    f"Fieldsets must have the same number of fields for this operation! {len(self.fields)} != {len(other)}"
+                )
+            with open(result.temporary.path, "wb") as fout:
+                for f, g in zip(self.fields, other.fields):
+                    sp = np.sqrt(np.square(f.values()) + np.square(g.values()))
+                    c = f.clone()
+                    c.encode_values(sp)
+                    param_id_u = f.grib_get("paramId", CodesHandle.LONG)
+                    param_id_sp = param_ids.get(param_id_u, None)
+                    if param_id_sp is not None:
+                        c = c.grib_set(["paramId", param_id_sp], CodesHandle.LONG)
+                    result._append_field(c)
+                    result.fields[-1].write(
+                        fout, result.temporary.path, temp=result.temporary
+                    )
+            return result
 
 
 class FieldsetCF:
