@@ -18,6 +18,7 @@ import xarray as xr
 
 import metview as mv
 from metview import bindings
+from metview.metviewpy import utils
 
 
 PATH = os.path.dirname(__file__)
@@ -46,6 +47,16 @@ def file_in_testdir(filename):
     return os.path.join(PATH, filename)
 
 
+def get_test_data(filename):
+    d_path = os.path.join(PATH, "data")
+    os.makedirs(d_path, mode=0o755, exist_ok=True)
+    f_path = os.path.join(d_path, filename)
+    if not os.path.exists(f_path):
+        URL = "https://get.ecmwf.int/repository/test-data/metview/tests"
+        utils.simple_download(url=f"{URL}/{filename}", target=f_path)
+    return f_path
+
+
 def test_version_info():
     out = mv.version_info()
     assert "metview_version" in out
@@ -103,6 +114,24 @@ def test_definitions_as_dict():
     assert mcont_dict["CONTOUR_MIN_LEVEL"] == 4.9
 
 
+def test_definitions_as_args():
+    mcont_def = mv.mcont(
+        legend=True,
+        contour_line_thickness=7,
+        contour_level_selection_type="level_list",
+        contour_level_list=[5, 7, 8.8, 9],
+        contour_hi_text="My high text",
+        CONTOUR_MIN_LEVEL=4.9,
+    )
+    mcont_dict = mcont_def.copy()
+    assert mcont_dict["LEGEND"] == "ON"
+    assert mcont_def["CONTOUR_LINE_THICKNESS"] == 7
+    assert mcont_def["CONTOUR_LEVEL_SELECTION_TYPE"] == "LEVEL_LIST"
+    assert mcont_def["CONTOUR_LEVEL_LIST"] == [5, 7, 8.8, 9]
+    assert mcont_def["CONTOUR_HI_TEXT"] == "My high text"
+    assert mcont_def["CONTOUR_MIN_LEVEL"] == 4.9
+
+
 def test_modify_request():
     c = mv.mcont(
         {
@@ -414,19 +443,19 @@ def test_division_fieldsets():
 
 
 def test_power():
-    raised_two = TEST_FIELDSET ** 2
+    raised_two = TEST_FIELDSET**2
     maximum = mv.maxvalue(raised_two)
-    assert np.isclose(maximum, MAX_VALUE ** 2)
+    assert np.isclose(maximum, MAX_VALUE**2)
 
 
 def test_power_reverse():
     mask = TEST_FIELDSET > 290
     FS_3_AND_4 = (mask * 3) + (1 - mask) * 4
-    raised = 2 ** FS_3_AND_4
+    raised = 2**FS_3_AND_4
     minimum = mv.minvalue(raised)
     maximum = mv.maxvalue(raised)
-    assert np.isclose(minimum, 2 ** 3)
-    assert np.isclose(maximum, 2 ** 4)
+    assert np.isclose(minimum, 2**3)
+    assert np.isclose(maximum, 2**4)
 
 
 def test_mod():
@@ -470,18 +499,60 @@ def test_distance():
     assert np.isclose(maximum, SEMI_EQUATOR)
 
 
-def test_fieldset_and():
+def test_fieldset_and_fieldset():
     t = (TEST_FIELDSET > 290) & (TEST_FIELDSET < 310)
     s = mv.accumulate(t)
     assert s == 43855
 
 
-def test_fieldset_or():
+def test_fieldset_and_bool():
+    t = (TEST_FIELDSET > 290) & (True)
+    s = mv.accumulate(t)
+    assert s == 45423
+    assert s == mv.accumulate(TEST_FIELDSET > 290)
+    t = (TEST_FIELDSET > 290) & (False)
+    s = mv.accumulate(t)
+    assert s == 0
+
+
+def test_bool_and_fieldset():
+    t = (True) & (TEST_FIELDSET > 290)
+    s = mv.accumulate(t)
+    assert s == 45423
+    assert s == mv.accumulate(TEST_FIELDSET > 290)
+    t = (False) & (TEST_FIELDSET > 290)
+    s = mv.accumulate(t)
+    assert s == 0
+
+
+def test_fieldset_or_fieldset():
     t = (TEST_FIELDSET < 250) | (TEST_FIELDSET > 310)
     s = mv.accumulate(t)
     assert s == 12427
 
 
+def test_fieldset_or_bool():
+    t = (TEST_FIELDSET < 250) | (True)
+    s = mv.accumulate(t)
+    assert s == 115680
+    assert s == len(TEST_FIELDSET.values())
+    t = (TEST_FIELDSET < 250) | (False)
+    s = mv.accumulate(t)
+    assert s == 10859
+    assert s == mv.accumulate(TEST_FIELDSET < 250)
+
+
+def test_bool_or_fieldset():
+    t = (True) | (TEST_FIELDSET < 250)
+    s = mv.accumulate(t)
+    assert s == 115680
+    assert s == len(TEST_FIELDSET.values())
+    t = (False) | (TEST_FIELDSET < 250)
+    s = mv.accumulate(t)
+    assert s == 10859
+    assert s == mv.accumulate(TEST_FIELDSET < 250)
+
+
 def test_fieldset_not():
     t = ~(TEST_FIELDSET < 250)
     s = mv.accumulate(t)
@@ -866,6 +937,14 @@ def test_datainfo():
     assert di3["proportion_missing"] == 0
 
 
+def test_grib_get_key_not_exist():
+    a = mv.read(os.path.join(PATH, "tuv_pl.grib"))[0:2]
+    kv = a.grib_get(["silly"])
+    assert kv == [[None], [None]]
+    with pytest.raises(Exception):
+        kv = a.grib_get_long(["silly"])
+
+
 def test_empty_fieldset_contructor():
     f = mv.Fieldset()
     assert type(f) == mv.Fieldset
@@ -1254,6 +1333,49 @@ def test_geopoints_nonequality_operator():
     assert vdiff[15] == 1
 
 
+def _optional_check_create_geo_inline():
+    # creates geopoints using numpy arrays and lists
+    # values are valid, np.nan and None
+    # - the ability to handle np.nan in lists is only available
+    #   in Metview 5.15.0 and above
+    g = mv.create_geo(
+        type="ncols",
+        latitudes=np.array([4, 5, np.nan]),
+        longitudes=[2.3, np.nan, 6.5],
+        levels=850,  # all rows will have 850 as their level
+        dates=[20180808, 20170707, 20160606],  # dates as numbers
+        times=None,
+        stnids=["aberdeen", "aviemore", "edinburgh"],
+        elevations=np.array([np.nan, 14.1, np.nan]),
+        temp=[273.15, np.nan, 281.45],
+        precip=[None, np.nan, 1],
+        speed=np.array([2, 3, 5]),
+    )
+
+    def check_columns(gpt):
+        aeq = np.testing.assert_array_equal
+        aeq(mv.latitudes(gpt), np.array([4, 5, np.nan]))
+        aeq(mv.longitudes(gpt), np.array([2.3, np.nan, 6.5]))
+        aeq(mv.levels(gpt), np.array([850, 850, 850]))
+        aeq(mv.elevations(gpt), np.array([np.nan, 14.1, np.nan]))
+        aeq(gpt["temp"], np.array([273.15, np.nan, 281.45]))
+        aeq(gpt["precip"], np.array([np.nan, np.nan, 1]))
+
+    check_columns(g)
+    temp_file = file_in_testdir("created_geo.gpts")
+    # check that it's written to disk ok
+    g.write(temp_file)
+    h = mv.read(temp_file)
+    check_columns(h)
+    os.remove(temp_file)
+
+
+def test_create_geo_inline():
+    version = mv.version_info()
+    if version["metview_version"] >= 51500.0:
+        _optional_check_create_geo_inline()
+
+
 def test_geopoints_set_dates():
     g = mv.create_geo(3)
 
@@ -1809,21 +1931,6 @@ def test_temporary_file_deletion(file_name):
     assert not (os.path.isfile(temp_filepath))
 
 
-def test_mvl_ml2hPa():
-    ml_data = mv.read(file_in_testdir("ml_data.grib"))
-    assert mv.type(ml_data) == "fieldset"
-    ml_t = mv.read(data=ml_data, param="t")
-    ml_lnsp = mv.read(data=ml_data, param="lnsp")
-    desired_pls = [1000, 900, 850, 500, 300, 100, 10, 1, 0.8, 0.5, 0.3, 0.1]
-    pl_data = mv.mvl_ml2hPa(ml_lnsp, ml_t, desired_pls)
-    assert mv.type(pl_data) == "fieldset"
-    pls = mv.grib_get_long(pl_data, "level")
-    lev_types = mv.grib_get_string(pl_data, "typeOfLevel")
-    lev_divisors = [1 if x == "isobaricInhPa" else 100 for x in lev_types]
-    pl_in_hpa = [a / b for a, b in zip(pls, lev_divisors)]
-    assert pl_in_hpa == desired_pls
-
-
 def test_push_nil():
     n = mv.nil()
     assert n is None
@@ -2363,3 +2470,79 @@ def test_download_gallery_data_bad_fname():
     fname = "zzz_for_spectra.grib"
     with pytest.raises(Exception):
         g = mv.gallery.load_dataset(fname)
+
+
+def test_speed():
+    # test with grib written with write() function
+    fs = mv.Fieldset(path=os.path.join(PATH, "tuv_pl.grib"))
+
+    fs_u = fs.select(shortName="u")
+    fs_v = fs.select(shortName="v")
+
+    # single field
+    u = fs_u[0]
+    v = fs_v[0]
+    r = mv.speed(u, v)
+    assert len(r) == 1
+    np.testing.assert_allclose(
+        r.values(), np.sqrt(np.square(u.values()) + np.square(v.values())), rtol=1e-05
+    )
+    assert r.grib_get_long("paramId") == 10
+
+    # multi fields
+    u = fs_u[:2]
+    v = fs_v[:2]
+    r = mv.speed(u, v)
+    assert len(r) == 2
+    for i in range(len(r)):
+        np.testing.assert_allclose(
+            r[i].values(),
+            np.sqrt(np.square(u[i].values()) + np.square(v[i].values())),
+            rtol=1e-05,
+        )
+    assert r.grib_get_long("paramId") == [10, 10]
+
+
+def test_mvl_ml2hpa():
+    # 1. test against reference fields
+
+    fs = mv.Fieldset(path=get_test_data("tq_ml137.grib"))
+    t = fs.select(shortName="t")
+    lnsp = fs.select(shortName="lnsp")
+
+    # this is not the same order as in t_ref!
+    pres = [100, 500, 925, 850, 1000]
+    t_ref = mv.Fieldset(path=get_test_data("ml2pl_ref.grib"))
+
+    r = mv.mvl_ml2hPa(lnsp, t, pres)
+
+    levels = r.grib_get_long("level")
+    np.testing.assert_allclose(levels, pres)
+
+    sh = r.grib_get_string("shortName")
+    assert sh == ["t"] * len(pres)
+
+    for p in pres:
+        np.testing.assert_allclose(
+            r.select(level=p).values(),
+            t_ref.select(level=p).values(),
+            rtol=1e-05,
+            err_msg=f"p={p}",
+        )
+
+    # 2. test pressure values
+
+    ml_data = mv.read(file_in_testdir("ml_data.grib"))
+    assert mv.type(ml_data) == "fieldset"
+    ml_t = mv.read(data=ml_data, param="t")
+    ml_lnsp = mv.read(data=ml_data, param="lnsp")
+    desired_pls = [1000, 900, 850, 500, 300, 100, 10, 1, 0.8, 0.5, 0.3, 0.1]
+
+    pl_data = mv.mvl_ml2hPa(ml_lnsp, ml_t, desired_pls)
+
+    assert mv.type(pl_data) == "fieldset"
+    pls = pl_data.grib_get_long("level")
+    lev_types = pl_data.grib_get_string("typeOfLevel")
+    lev_divisors = [1 if x == "isobaricInhPa" else 100 for x in lev_types]
+    pl_in_hpa = [a / b for a, b in zip(pls, lev_divisors)]
+    assert pl_in_hpa == desired_pls
